/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
extern "C"
{
#if defined(SOLARIS) || defined(IRIX)
   #define HAVE_SYSINFO 2
   #include <sys/systeminfo.h>
#elif defined(OSF1)
   #define HAVE_SYSINFO 2
   #include <sys/systeminfo.h>
   extern sysinfo(int, char *, long);
#elif defined(HPUX) || defined(AIX) || defined(LINUX)
   #define HAVE_SYSINFO 1
   #include <sys/utsname.h>
#elif defined(XP_WIN32)
   #define HAVE_SYSINFO 3
   #include <windows.h>
#else
   #define HAVE_SYSINFO 0
#endif

#include <ctype.h>
}

#include "misc.h"

SysInfo::SysInfo()
{
   NSString osVersion  = "Unknown";
   NSString hwPlatform = "Unknown";
   char buf[MED_BUF];

#ifdef XP_WIN32
   SYSTEM_INFO si;
   OSVERSIONINFO osvi;
#endif

#if HAVE_SYSINFO == 2
   if (sysinfo(SI_SYSNAME, buf, MED_BUF) > 0)
   {
      osVersion = buf;
      if (sysinfo(SI_RELEASE, buf, MED_BUF) > 0)
      {
         osVersion = osVersion + " " + buf;
      }
      if (sysinfo(SI_VERSION, buf, MED_BUF) > 0)
      {
         osVersion = osVersion + " " + buf;
      }
   }

   if (sysinfo(SI_MACHINE, buf, MED_BUF) > 0)
   {
      hwPlatform = buf;
      if (sysinfo(SI_ARCHITECTURE, buf, MED_BUF) > 0)
      {
         hwPlatform = hwPlatform + " " + buf;
      }
#if defined(SOLARIS)
      if (sysinfo(SI_PLATFORM, buf, MED_BUF) > 0)
      {
         hwPlatform = hwPlatform + " " + buf;
      }
#endif
   }
#elif HAVE_SYSINFO == 1
   struct utsname sysname;
   if (uname(&sysname) >= 0)
   {
      osVersion = sysname.sysname;
      osVersion = osVersion + " " +  sysname.release;
      osVersion = osVersion + " " +  sysname.version;
      hwPlatform = sysname.machine;
   }
#elif HAVE_SYSINFO == 3
   GetSystemInfo(&si);
   switch (si.wProcessorArchitecture)
   {
   case PROCESSOR_ARCHITECTURE_INTEL:
      hwPlatform = "Intel";
      break;
   case PROCESSOR_ARCHITECTURE_MIPS:
      hwPlatform = "MIPS";
      break;
   case PROCESSOR_ARCHITECTURE_ALPHA:
      hwPlatform = "Alpha";
      break;
   case PROCESSOR_ARCHITECTURE_PPC:
      hwPlatform = "PowerPC";
      break;
   default:
      break;
   }
   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
   GetVersionEx(&osvi);

   wsprintf(buf, "Windows %s%ld.%ld (Build %d)",
            osvi.dwPlatformId == VER_PLATFORM_WIN32_NT ? "NT" : "",
            osvi.dwMajorVersion, osvi.dwMinorVersion,
            osvi.dwPlatformId == VER_PLATFORM_WIN32_NT ? osvi.dwBuildNumber:LOWORD(osvi.dwBuildNumber));

   osVersion = buf;

#endif

   set("osVersion", osVersion);
   set("hardwarePlatform", hwPlatform);
}
