/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/***********************************************************************
** NAME
**   dl-dnld.cc
**
** DESCRIPTION
**   Common dialog routines for Downloader
**
** NOTES:
**
** HISTORY
**
** $Log: dl-dnld.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:30  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 18:24:10  nhosoi
** rebranded setupsdk as brandx; updated the version number to 7.0
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.3  2001/09/20 21:36:36  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.2  2000/03/06 00:57:45  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.1  1999/03/04 23:27:00  svbld
** importing setupsdk41 closer to the trunk
**
** Revision 1.1.2.18.4.2  1999/01/16 05:59:21  cxue
** change Server Family to Server Products
**
** Revision 1.1.2.18.4.1  1998/10/29 03:13:34  cxue
** move dialog resource from dl-common.cc to uxres_data.cc
**
** Revision 1.1.2.18  1998/07/16 17:12:22  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.17  1998/05/29 16:55:13  pvo
** 117356 - component selection accepts any selection
**
** Revision 1.1.2.16  1998/05/01 23:24:40  gbeasley
** 113120 - New termonologies
**
** Revision 1.1.2.15  1998/03/24 22:29:44  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.14  1998/03/10 02:49:13  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.13  1998/03/09 22:09:35  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.12  1998/03/05 03:47:34  pvo
** Fixed build problem.
**
** Revision 1.1.2.11  1998/03/04 23:03:53  pvo
** Set up correct platform default for download.
**
** Revision 1.1.2.10  1998/03/02 23:59:25  pvo
** 106912 - Pickup the right defaults in dialogs.
** 105320 - Improve logging of silent installation.
**
** Revision 1.1.2.9  1998/02/26 00:13:58  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.8  1998/02/17 23:36:49  pvo
** Fixed problem with defaults for Yes/No answer.
**
** Revision 1.1.2.7  1998/02/15 22:06:38  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.6  1998/01/22 04:51:26  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.5  1998/01/09 03:27:27  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.4  1997/12/12 21:54:15  pvo
** Various bug fixes.
**
** Revision 1.1.2.3  1997/12/02 20:39:07  pvo
** Moved suitespot.inf to ./inf directory.
** Fixed problem with ns-config being called twice.
** Silent Installation support.
**
** Revision 1.1.2.2  1997/12/02 04:28:26  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.1  1997/11/26 02:02:53  pvo
** Restructure files and add Download support.
**
**
***********************************************************************/

extern "C"
{
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
}

#include "ux-util.h"
#include "dialog.h"
#include "dl-common.h"
#include "dl-dnld.h"
#include "downloader.h"


/*
** Forward References
**/

DialogAction welcomeNext (Dialog *me);
DialogAction askOptionsSetup(Dialog *me);
DialogAction askOptionsNext(Dialog *me);
DialogAction askSourceOptionsSetup(Dialog *me);
DialogAction askSourceOptionsNext(Dialog *me);
DialogAction askNetscapeSourceSetup(Dialog *me);
DialogAction askNetscapeSourceNext(Dialog *me);
DialogAction askIntranetSourceNext(Dialog *me);
DialogAction askIntranetSourceSetup(Dialog *me);
DialogAction askPlatformSetup(Dialog *me);
DialogAction askPlatformNext(Dialog *me);
/*
 * Ask components
 */
DialogAction askDestinationSetup(Dialog *me);
DialogAction askDestinationNext(Dialog *me);

void
setupSelectionDialog (Dialog *me, const NVPair *p)
{
   char tstr[SML_BUF];
   NSString text2;
   const char *s;

   for (int i = 0; i < p->numName(); i++)
   {
      s = p->name(i);
      sprintf(tstr, "   %d. %s\n", i+1, s);
      text2 = text2 + tstr;
   }
   me->setText2(text2.data());
}

const char *
getSelectionValue (const NVPair *selections, int which)
{
   const char *val = NULL;

   if (which <= selections->numName() && which > 0)
   {
      which--;
      for (int i = 0; i < selections->numName(); i++)
      {
         val = selections->name(i);
         if (i == which)
         {
            val = selections->get(val); 
            break;
         }
      }
   }
   return val;
}

void
downloaderError(const InetError & err)
{
   DialogAlert alert(err.msg());
   alert.execute();
}

/////////////////////////////////////////////////////////////////////////
/// Welcome 
/////////////////////////////////////////////////////////////////////////

DialogYesNo welcome(
NULL, /* uxres_data:RES_DIALOG_WELCOME_TEXT[0,1] */

// "Would you like to continue with setup?"

NULL, /* uxres_data:RES_DIALOG_WELCOME_TEXT2 */

"Yes",

welcomeSetup,
welcomeNext
);

DialogAction welcomeNext (Dialog *me)
{
   const char *buf = me->input();
   DialogAction rc;

   if (buf[0] == 0)
   {
       buf = me->defaultAns();
   }
   rc = yesNoDefaultYes(buf) ? DIALOG_NEXT : DIALOG_EXIT;
   if (rc == DIALOG_NEXT && !me->isVisited())
   {
      askOptions.registerDialogNext(me->manager());
      me->visited();
   }
   return rc;
}


////////////////////////////////////////////////////////////////////////
/// AskOptions
/////////////////////////////////////////////////////////////////////////

DialogInput askOptions(
NULL, /* RES_DIALOG_ASKOPTIONS[0] */

"Choose your installation type",

DOWNLOAD_ONLY,

askOptionsSetup,
askOptionsNext
);

DialogAction askOptionsSetup(Dialog *me)
{
   NSString askoptions_text(getResource(RES_DIALOG_ASKOPTIONS, RES_GLOBAL_BRAND));
   askoptions_text = askoptions_text + 
                     getResource(RES_DIALOG_ASKOPTIONS0, RES_GLOBAL_BRAND);
   me->setText(askoptions_text);
   dialogSetup(me, DOWNLOAD_OPTION);
   return DIALOG_SAME;
}

DialogAction askOptionsNext(Dialog *me)
{
   const char *buf = me->input();
   const char *ans;
   Downloader *downloader = (Downloader *) me->manager()->parent();
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   DialogAction rc ;

   if (buf[0] == 0 )
   {
       ans = me->defaultAns();
   }
   else
   {
       ans = buf;
   }

   if (strcmp(ans, DOWNLOAD_ONLY) && strcmp(ans, DOWNLOAD_AND_INSTALL))
   {
      rc = DIALOG_SAME;
   }
   else if (!me->isVisited() || strcasecmp(ans, installInfo->get(DOWNLOAD_OPTION)))
   {
      if (!strcmp(ans, DOWNLOAD_ONLY))
      {
         installInfo->set(PLATFORM, "");
      }
      else 
      {
         installInfo->set(PLATFORM, InstUtil::getSysVersion());
      }

      if (!me->isVisited())
      {
         askSourceOptions.registerDialogNext(me->manager());
         me->visited();
      }

      installInfo->set(DOWNLOAD_OPTION, ans);

      rc = DIALOG_NEXT;
   }

   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// AskSourceOptions
/////////////////////////////////////////////////////////////////////////
DialogInput askSourceOptions(
NULL, /* RES_DIALOG_ASKSOURCEOPTIONS[0] */

"Download location",

FROM_NETSCAPE,

askSourceOptionsSetup,
askSourceOptionsNext
);

DialogAction
askSourceOptionsSetup(Dialog *me)
{
   NSString asksoptions_text(getResource(RES_DIALOG_ASKSOURCEOPTIONS));
   asksoptions_text = asksoptions_text +
                   getResource(RES_DIALOG_ASKSOURCEOPTIONS0, RES_GLOBAL_VENDOR);
   me->setText(asksoptions_text);
   dialogSetup(me, DOWNLOAD_SOURCE);
   return DIALOG_SAME;
}

DialogAction
askSourceOptionsNext (Dialog *me)
{
   NSString ans;
   NSString errMsg; 
   Downloader *downloader = (Downloader *) me->manager()->parent();
   InetError err;
   DialogAction rc = DIALOG_NEXT;
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   const char *buf = me->input();

   if (buf[0] == 0)
   {
      ans = me->defaultAns();
   }
   else
   {
      ans = buf;
   }
   /*
    * Either Netscape(1) or Intranet (2)
    */

   if (strcasecmp(ans, FROM_NETSCAPE) && strcasecmp(ans, FROM_INTRANET))
   {
      rc = DIALOG_SAME;
   }
   else if (me->isVisited() == False || strcasecmp(ans, installInfo->get(DOWNLOAD_SOURCE)))
   {
      if (!strcasecmp(ans, FROM_NETSCAPE))
      {
         // Retrieve site information file
         installInfo->set(NETSCAPE_SOURCE_URL, NETSCAPE_URL);
         downloader->setSource(NETSCAPE_URL);

         err = downloader->retrieveSiteInfo();
         if (err != INET_OK)
         {
            downloaderError(err);
            rc = DIALOG_SAME;
         }
         else
         {
            askNetscapeSource.registerDialogNext(me->manager());
         }
      }
      else if (!strcmp(ans, FROM_INTRANET))
      {
         askIntranetSource.registerDialogNext(me->manager());
      }
      else
      {
         rc = DIALOG_SAME;
      }
   }

   if (rc == DIALOG_NEXT)
   {
      me->visited();
      installInfo->set(DOWNLOAD_SOURCE, ans);
   }

   return rc;

}
/////////////////////////////////////////////////////////////////////////
/// AskSource
/////////////////////////////////////////////////////////////////////////
DialogInput askNetscapeSource(
NULL,  /** RES_DIALOG_ASKVSOURCEOPTIONS **/
"FTP server",
"1",   /* Default to the first one */
askNetscapeSourceSetup,
askNetscapeSourceNext
);

DialogAction
askNetscapeSourceSetup(Dialog *me)
{
   const NVPair *siteInfo;
   NSString askvsoptions_text(getResource(RES_DIALOG_ASKVSOURCEOPTIONS, RES_GLOBAL_VENDOR));
   me->setText(askvsoptions_text);
   siteInfo = ((Downloader *) me->manager()->parent())->getSiteInfo();

   setupSelectionDialog(me, siteInfo);

   return DIALOG_SAME;
}

DialogAction
askNetscapeSourceNext(Dialog *me)
{
   Downloader *downloader = (Downloader *) me->manager()->parent();
   const NVPair *siteInfo = downloader->getSiteInfo();
   const char *buf = me->input();
   const char *ans;
   int which;
   InetError err;
   DialogAction rc = DIALOG_NEXT;
   InstallInfo *installInfo=me->manager()->getDefaultScript();
   int needPlatform;

   if (buf[0] == 0)
   {
     ans = me->defaultAns();
   }
   else
   {
     ans = buf;
   }

   me->setDefaultAns(ans);

   needPlatform = !strcmp(installInfo->get(PLATFORM),"");

   which = atoi(ans);

   ans = getSelectionValue(siteInfo, which);

   if (ans == NULL)
   {
       rc = DIALOG_SAME;
   }
   else if (me->isVisited() == False || strcasecmp(ans, installInfo->get(SOURCE_URL)) || needPlatform)
   {
      // Not visited yet or source URL has been changed or download option
      // has been changed

      installInfo->set(SOURCE_URL,ans);
      downloader->setSource(ans);

      if (needPlatform)
      {
         err = downloader->retrievePlatformInfo();
         if (err != INET_OK)
         {
            installInfo->set(SOURCE_URL, "");
            downloaderError(err);
            rc = DIALOG_SAME;
         }
      }
   }

   if (rc == DIALOG_NEXT)
   {
      // If not visited yet or already visited but still need to get
      // platform information (since user changed from DownloadAndInstall
      // to DownloadOnly), ask for platform information. 
      if (needPlatform)
      {
         askPlatform.registerDialogNext(me->manager());
      }
      else if (me->isVisited() == False)
      {
         err = downloader->retrieveDownloadInfo();

         if (err != INET_OK)
         {
            installInfo->set(PLATFORM, "");
            downloaderError(err);
            rc =  DIALOG_EXIT;
         }
      }
      if (rc == DIALOG_NEXT && !needPlatform)
      {
         Dialog *selectionDialog;
         me->visited();
         selectionDialog = createSelectionDialog(
               downloader->suitespot()->getComponents(),
               "Specify the components you wish to download",
               askSelectionNext,
           getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND));
         selectionDialog->registerDialogLast(me->manager());
      }
   }

   return rc;

}

/////////////////////////////////////////////////////////////////////////
/// AskIntranetSource
/////////////////////////////////////////////////////////////////////////
DialogInput askIntranetSource(
"Please specify the URL for your Intranet FTP location in the form\n"
"similar to the below:\n"
"\n"
"   ftp://<hostname>/path",
"URL of FTP location",

NULL,

askIntranetSourceSetup,
askIntranetSourceNext
);

DialogAction
askIntranetSourceSetup(Dialog *me)
{
   dialogSetup(me, SOURCE_URL);
   return DIALOG_SAME;
}

DialogAction
askIntranetSourceNext(Dialog *me)
{
   const char *ans, *buf = me->input();
   InetError err;
   Downloader *downloader = (Downloader *) me->manager()->parent();
   InstallInfo *installInfo = me->manager()->getDefaultScript();
   DialogAction rc = DIALOG_NEXT;
   int needPlatform;

   if (buf[0] == 0 )
   {
       ans = me->defaultAns();
   }
   else
   {
       ans = buf;
   }

   needPlatform = !strcmp(installInfo->get(PLATFORM), "");

   if (me->isVisited() == False || strcasecmp(ans, installInfo->get(SOURCE_URL)) || !strcmp(installInfo->get(PLATFORM),""))
   {
      installInfo->set(INTRANET_SOURCE_URL, ans);
      installInfo->set(SOURCE_URL, ans);
      downloader->setSource(ans);

      if (needPlatform)
      {
         err = downloader->retrievePlatformInfo();
         if (err != INET_OK)
         {
            installInfo->set(SOURCE_URL, "");
            downloaderError(err);
            rc = DIALOG_SAME;
         }
      }
   }
   if (rc == DIALOG_NEXT)
   {
      installInfo->set(SOURCE_URL, ans);
      if (me->isVisited() == False || needPlatform)
      {
         if (needPlatform)
         {
            askPlatform.registerDialogNext(me->manager());
         }
         else
         {
            err = downloader->retrieveDownloadInfo();

            if (err != INET_OK)
            {
               installInfo->set(PLATFORM, "");
               downloaderError(err);
               rc =  DIALOG_EXIT;
            }
            else
            {
               Dialog *selectionDialog;
               selectionDialog = createSelectionDialog(
                  downloader->suitespot()->getComponents(),
                  "Specify the components you wish to download",
                  askSelectionNext,
                  getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND));
               selectionDialog->registerDialogLast(me->manager());
            }
         }

         me->visited();
      }
   }

   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// AskPlatform
/////////////////////////////////////////////////////////////////////////
DialogInput askPlatform(
"Please choose which platform:\n"
"\n"
"To accept the default show in brackets, press the Return key."
"\n",
"Software platform",
NULL,
askPlatformSetup,
askPlatformNext
);

DialogAction
askPlatformSetup(Dialog *me)
{
   const NVPair *platformInfo = ((Downloader *) me->manager()->parent())->getPlatformInfo();
   const char *sysVersion = me->manager()->getDefaultScript()->get(SYS_VERSION);
   int i, j;
   Bool found = False;
   char tmp[5];

   setupSelectionDialog(me, platformInfo);

   j = 1;
   if (me->defaultAns() == NULL)
   {
      for (i = 0; i < platformInfo->numName(); i++)
      {
         const char *platformName = platformInfo->get(platformInfo->name(i));
         if (!strcasecmp(platformName, sysVersion))
         {
            found = True;
            break;
         }
         else if (!strncasecmp(platformName, sysVersion, 4))
         {
            j = i + 1;
         }
      }

      if (found)
      {
        j = i +1;
      }

      sprintf(tmp, "%d", j);
      me->setDefaultAns(tmp);
   }

   return DIALOG_SAME;
}

DialogAction
askPlatformNext(Dialog *me)
{
   Downloader *downloader = (Downloader *) me->manager()->parent();
   const NVPair *platformInfo = downloader->getPlatformInfo();
   const char *buf = me->input();
   const char *ans;
   int which;
   InetError err;
   DialogAction rc = DIALOG_NEXT;
   InstallInfo *installInfo = me->manager()->getDefaultScript();

   if (buf[0] == 0)
   {
     ans = me->defaultAns();
   }
   else
   {
     ans = buf;
   }

   which = atoi(ans);

   me->setDefaultAns(ans);

   ans = getSelectionValue(platformInfo, which);
   if (ans == NULL)
   {
       rc =  DIALOG_SAME; 
   }
   else if (me->isVisited() == False || strcasecmp(ans, installInfo->get(PLATFORM)))
   {
      installInfo->set(PLATFORM,ans);

      err = downloader->retrieveDownloadInfo();

      if (err != INET_OK)
      {
         installInfo->set(PLATFORM, "");
         downloaderError(err);
         rc =  DIALOG_SAME;
      }
   }
   if (rc == DIALOG_NEXT)
   {
      if (me->isVisited() == False)
      {
         Dialog *selectionDialog;
         selectionDialog = createSelectionDialog(
            downloader->suitespot()->getComponents(),
            "Specify the components you wish to download",
            askSelectionNext,
            getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND));
         selectionDialog->registerDialogLast(me->manager());

         me->visited();
      }
   }

   return rc;
}

/////////////////////////////////////////////////////////////////////////
/// AskDestination
/////////////////////////////////////////////////////////////////////////

DialogInput askDestination(
NULL,  /** RES_DIALOG_ASKDESTINATION **/

"Destination directory",

DEFAULT_DNLD_ROOT,

askDestinationSetup,
askDestinationNext
);

DialogAction askDestinationSetup(Dialog *me)
{
   NSString askdest_text(getResource(RES_DIALOG_ASKDESTINATION, RES_GLOBAL_BRAND));
   me->setText(askdest_text);
   dialogSetup(me, DOWNLOAD_ROOT);
   return DIALOG_SAME;
}

DialogAction askDestinationNext(Dialog *me)
{
   int err = 0;
   const char *ans;
   char errMsg[BIG_BUF];
   Downloader *downloader = (Downloader *) me->manager()->parent();
   DialogAction rc = DIALOG_EXIT; // Last dialog
   InstallInfo *installInfo=me->manager()->getDefaultScript();

   const char *buf = me->input();

   if (buf[0] == 0)
   {
      ans = me->defaultAns();
   }
   else
   {
      ans = buf;
   }

   if (me->isVisited() == False || strcasecmp(ans, installInfo->get(DOWNLOAD_ROOT)))
   {
      installInfo->set(DOWNLOAD_ROOT, ans);
      rmdir (ans);

      if (InstUtil::dirExists(ans) == False)
      {
         err = InstUtil::createDirectory(ans, 0755);
      }

      if (err)
      {

         if (err == -1)
         {
            sprintf(errMsg, "ERROR: Can't create directory %s\n", ans);
         }
         else if (err == -2)
         {
            sprintf(errMsg, "ERROR: %s exists and is not a directory.\n", ans);
         }
         else if (err == -3)
         {
            sprintf(errMsg, "ERROR: Can't write to %s\n", ans);
         }

         DialogAlert alert(errMsg);

         alert.execute();

         rc = DIALOG_SAME;
      }
   }

   if (rc == DIALOG_EXIT)
   {
      downloader->setDestination(ans);
      me->visited();
   }

   return rc;
}
   
DialogAction
verifySelection (Dialog *me)
{
   Downloader  *downloader = (Downloader *) me->manager()->parent();
   Components *components;

   me->getUserData("Components", (long &) components);
   if (getSelectedComponent (me, components, False) == 0)
   {
      return DIALOG_SAME;
   }
   return DIALOG_NEXT;
}

DialogAction
askSelectionNext(Dialog *me)
{
   DialogAction rc;
   rc = verifySelection(me);

   if (rc == DIALOG_NEXT)
   {
      
      /*
       * Take care of module
       */
      Components *products;
      me->getUserData("Components", (long &) products);

      if (products->isTop())
      {
         me->manager()->resetLast();
      }

      for (int i = 0 ; i < products->numComponents(); i++)
      {
         Product *p = products->getComponent(i);

         if (p->isSelected() && p->numComponents() > 0 )
         {
            Dialog *selectionDialog;
            selectionDialog = createSelectionDialog(
               p->getComponents(),
               "Specify the components you wish to download",
               askSelectionNext,
               getResource(RES_DIALOG_SELECTION_TEXT, RES_GLOBAL_BRAND),
               p->get(PKG_NICKNAME));
            selectionDialog->registerDialogLast(me->manager());

         }
      }
      me->visited();
      askDestination.registerDialogLast(me->manager());
   }
   return rc;
}
