/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/**********************************************************************
** NAME
**   dialog.cc
**
** DESCRIPTION
**   Implementation of Dialog related classes.
**
** NOTES
**
** HISTORY
**
** $Log: dialog.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.4  2005/07/29 19:43:30  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.3  2005/03/01 01:45:18  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/09 01:33:38  nhosoi
** bugzilla 147447: De-brand the Setup SDK as "brandx"; attachment (110842)
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.8  2001/09/20 21:37:27  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.7  1999/11/09 00:04:50  cxue
** netscape to alliance
**
** Revision 1.1.1.6  1999/10/01 20:54:19  cxue
** Simple YesNoDialog for warning
**
** Revision 1.1.1.5  1999/09/17 21:21:50  cxue
** Move inline funciton -- answer() to normal function to solve compilation error
**
** Revision 1.1.1.4  1999/07/29 00:24:00  svbld
** Merged the admin42_branch back onto the admin_branch..
**
** Revision 1.1.1.2.2.1  1999/06/16 22:47:07  cxue
** Calling fgets instead of gets to get rid of warnings on Linux
**
** Revision 1.1.1.2  1999/04/22 21:41:51  cxue
** bug 342666. non-conversion for normal input field
**
** Revision 1.1.2.15.6.5  1998/10/19 21:12:46  cxue
** pass default value to curses
**
** Revision 1.1.2.15.6.4  1998/10/15 22:22:09  cxue
** resource framework for custom ui
**
** Revision 1.1.2.15.6.3  1998/10/13 21:05:34  cxue
** testing NULL pointer in setText,setText2,setPrompt,setDefaultAns
**
** Revision 1.1.2.15.6.2  1998/10/02 18:05:40  cxue
** Support mode=express|typical|custom|all modes in [general] section. New hidden attribute for Dialog class
**
** Revision 1.1.2.15.6.1  1998/09/11 23:52:27  cxue
** 323376. Change initscr to newterm, so we can have fallback mechanism in case TERM value is not correct
**
** Revision 1.1.2.15  1998/06/10 21:14:11  pvo
** Allow returning DIALOG_PREV from _setupAction
**
** Revision 1.1.2.14  1998/05/08 02:15:03  pvo
** 116843 - check for TERM
**
** Revision 1.1.2.13  1998/04/14 01:11:59  pvo
** 113369 - added pause
**
** Revision 1.1.2.12  1998/03/28 03:29:35  pvo
** 103621 - Support for 8-bit input.
**
** Revision 1.1.2.11  1998/03/24 22:29:53  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.10  1997/12/02 04:28:33  pvo
** Support for Silent installation and various bug fixes.
**
** Revision 1.1.2.9  1997/11/26 01:11:48  pvo
** Added enable/disable back screen capability.
**
** Revision 1.1.2.8  1997/11/13 19:28:54  pvo
** More new stuffs.
**
** Revision 1.1.2.7  1997/11/08 16:57:17  pvo
** Added Dialog::newPage() for general use.
**
** Revision 1.1.2.6  1997/10/24 03:39:42  pvo
** Final modification to make all work across platforms! Hope I make
** it this time.
**
** Revision 1.1.2.5  1997/10/23 03:23:43  pvo
** More change to support other platforms.
**
** Revision 1.1.2.4  1997/10/22 23:59:01  pvo
** Warning removal project!
**
** Revision 1.1.2.3  1997/10/22 02:40:54  pvo
** Too many changes to list out here.
**
** Revision 1.1.2.2  1997/09/30 18:40:46  pvo
** More documentation.
**
** Revision 1.1.2.1  1997/09/27 02:47:30  pvo
** Checked in.
**
***********************************************************************/

/*
 * Includes
 */
#include "utf8.h"
#include "dialog.h"

extern "C" {
#include "ux-curse.h"
}

//
// CLASS: DialogManager
//
DialogManager::DialogManager()
   :_installMode(Interactive)
{
}

DialogManager::~DialogManager()
{
}

Bool DialogManager::_winMode = False;

void
DialogManager::enableWinMode()
{
   if (_winMode == False)
   {
      w_initscr();
      _winMode = True;
   }
   else
   {
      w_refresh();
   }
}

void
DialogManager::disableWinMode()
{
   if (_winMode == True)
   {
      w_endwin();
   }
}

void
DialogManager::newPage()
{
   if (_winMode == True)
   {
      new_page();
   }
}

void
DialogManager::setDefaultScript(InstallInfo *script)
{
   _installInfo = script;
}

InstallInfo *
DialogManager::getDefaultScript() const
{
   return _installInfo;
}

NSString Dialog::_title = "Fedora Project";
NSString Dialog::_product = "Fedora Server Products";

//
// CLASS: Dialog
//
Dialog::Dialog (const char *text, const char *prompt, const char *defaultAns,
           DLG_CALLBACK setup, DLG_CALLBACK next, DLG_CALLBACK prev)
   :_text(text), _prompt(prompt),_defaultAns(defaultAns),
    _nextAction(next), _prevAction(prev), _setupAction(setup),
    _bufLen(sizeof(_buf)), _visited(False),_enable8BitInput(False),
    _hidden(False)
{
}

void
Dialog::initDisplay(const char *what, const char *title)
{
   if (title)
      _title = title;

   if (what)
      _product = what;

   init_screen(what, _title);
}

void
Dialog::initDisplay(const char *what)
{
   if (what)
      _product = what;

   init_screen(_product, _title);
}

void
Dialog::setText(const char *text)
{
   if (text)
      _text = text;
}

void
Dialog::setText2(const char *text)
{
   if (text)
      _text2 = text;
}

void
Dialog::setPrompt (const char *prompt)
{
   if (prompt)
      _prompt = prompt;
}

void
Dialog::setDefaultAns(const char *ans)
{
   if (ans)
      _defaultAns = ans;
}

const char *
Dialog::answer () const
{
	if (_buf[0] == '\0')
		return _defaultAns;
	else
		return _buf;
}

void
Dialog::setInputLen(int length)
{
   _bufLen = length;
}

void
Dialog::setSetupAction(DLG_CALLBACK f)
{
   _setupAction = f;
}

void
Dialog::setNextAction(DLG_CALLBACK f)
{
   _nextAction = f;
}

void
Dialog::setPrevAction(DLG_CALLBACK f)
{
   _prevAction = f;
}

void
Dialog::setUserData(const char *name, const char *value)
{
   _userData.set(name,value);
}

void
Dialog::setUserData(const char *name, long value)
{
   _userData.set(name,value);
}

const char *
Dialog::getUserData(const char *name)
{
   return _userData.get(name);
}

int
Dialog::getUserData(const char *name, long & value)
{
   const char *s;

   s = _userData.get(name);
   if (s)
   {
      sscanf(s, "%ld", &value);
      return 0;
   }
   return -1;
}

DialogAction
Dialog::next()
{
   DialogAction rc = DIALOG_NEXT;
   if (_nextAction)
   {
      rc = _nextAction(this);
   }
   return rc;
}

int
Dialog::getInput()
{
   int rc;
   char *utf8Buf = NULL;

   rc =  grab_string(_buf, _bufLen, _defaultAns.data());
   if (rc > 0)
   {
      if (_enable8BitInput)
      {
         utf8Buf = localToUTF8(_buf);
         strcpy(_buf, utf8Buf);
         free(utf8Buf);
      }
   }
   return rc;
} 

int
Dialog::getPassword()
{
   return (grab_pw_string(_buf, _bufLen, _defaultAns.data()));
}

void
Dialog::prev()
{
   if (_prevAction)
   {
       _prevAction(this);
   }
}

DialogAction 
Dialog::show()
{
    DialogAction rc = DIALOG_SAME;

    if (_setupAction)
    {
      rc = _setupAction(this);
    }

    if (rc != DIALOG_SAME)
       return rc;

    if (isHidden() == True)
       return DIALOG_NEXT;

    new_page();

    if (_text.data())
    {
       printw((char *)_text.data());
       printw("\n\n");
    }

    if (_text2.data())
    {
       printw((char *)_text2.data());
       printw("\n\n");
    }

    if (_prompt.data())
    {
       char tstr[BIG_BUF];
       printw((char *)_prompt.data());

       if (_defaultAns != (char *) NULL)
       {
          char *ans;
          if (_enable8BitInput)
            ans = UTF8ToLocal(_defaultAns.data());
          else
            ans = strdup((const char *) _defaultAns);
          sprintf(tstr, " [%s]: ", ans);
          free(ans);
       }
       else
       {
          sprintf(tstr, "%c ", ':');
          _buf[0] = 0;
       }
       printw(tstr);
   }

   if (getInput())
   {
      rc = DIALOG_NEXT;
   }
   else
   {
      rc =  DIALOG_PREV;
   }
   return rc;
}

void
Dialog::showText()
{
   new_page();

   if (_text.data())
   {
      printw((char *)_text.data());
      printw("\n");
   }
   if (_text2.data())
   {
      printw((char *)_text2.data());
      printw("\n");
   }

   show_message("");
}

void
Dialog::showString(const char *str)
{
   if (str)
   {
      printw((char *)str);
   }
   w_refresh();
} 

void
Dialog::showProgress(const char *str)
{
   if (str)
   {
      print_oneplace((char *) str);
      w_refresh();
   }
}

void
Dialog::pause(const char *str)
{
   char buf[XSM_BUF];
   const char *msg;

   if (str)
   {
      msg = str;
   }
   else
   {
      msg = "Press Return to continue...";
   }
   if (w_isendwin())
   {
      printf("\n");
      printf((char *) msg);
      printf("\n");
      fflush(stdout);
      fgets(buf, sizeof(buf), stdin);
   }
   else
   {
      printw("\n");
      printw((char *) msg);
      printw("\n");
      grab_string(buf, XSM_BUF, NULL);
   }
}

void
Dialog::disableBackPage()
{
   disableScreenBack();
}

void
Dialog::registerDialogNext(DialogManager *dlgMgr)
{
   _dlgMgr = dlgMgr;
   dlgMgr->add(this);
}
void
Dialog::registerDialogLast(DialogManager *dlgMgr)
{
   _dlgMgr = dlgMgr;
   dlgMgr->addLast(this);
}

//
// CLASS: DialogYesNo
//
DialogYesNo::DialogYesNo(const char *text, const char *prompt, const char *defaultAns, DLG_CALLBACK setup, DLG_CALLBACK next, DLG_CALLBACK prev)
   :Dialog(text,prompt,defaultAns,setup,next,prev)
{
  _default = False;
   setInputLen(5);
}

DialogYesNo::DialogYesNo(const char *text, const char *defaultAns)
   :Dialog(text,"Do you want to continue",defaultAns)
{
   setInputLen(5);   
  _default = True;
}

DialogAction DialogYesNo::processInput()
{
  const char *buf = answer();
  char input;

  input = buf[0];

  if (input == 'Y' || input == 'y')
    return DIALOG_NEXT;
  else if (input == 'N' || input == 'n')
    return DIALOG_SAME;

}


DialogAction
DialogYesNo::execute()
{
   /*
    * Execute returns:
    *   1: OK to continue
    *   0: OK to go back
    *  -1: error, to exit
    */
   DialogAction rc;

   rc = Dialog::show();

   if (rc == DIALOG_NEXT)
   {
     if (_default == True)
       rc = processInput();
     else
        rc = next();
   }
   else
   {
      prev();
   } 

   return rc;
}

//
// CLASS: DialogInput
//
DialogInput::DialogInput(const char *text, const char *prompt, const char *defaultAns, DLG_CALLBACK setup, DLG_CALLBACK next, DLG_CALLBACK prev, Bool allow8BitInput)
   : Dialog(text,prompt,defaultAns,setup,next,prev)
{
   _enable8BitInput = allow8BitInput;
}

DialogAction
DialogInput::execute()
{
   DialogAction rc;

   rc = Dialog::show();
   
   if (rc == DIALOG_NEXT)
   {
      rc = next();
   }
   else
   {
      // Go back
      prev();
   }

   return rc;

}

//
// CLASS: DialogAlert
//
DialogAlert::DialogAlert(const char *text)
  :Dialog(text)
{
}

DialogAction
DialogAlert::execute()
{
   showText();
   return DIALOG_SAME;
}

void
DialogProgress::showProgress(const char *text)
{
   if (_newPage == False)
   {
      new_page();
      _newPage = True;
   }
   printw((char *)text);
   w_refresh();
}
DialogAction
DialogProgress::execute()
{
   return DIALOG_SAME;
}
