/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*********************************************************************
** NAME:
**   product.cc
**
** DESCRIPTION:
**   Definitions for Product class
**
** NOTES:
**
** HISTORY:
** $Log: product.cc,v $
** Revision 1.1.1.1  2005/07/29 22:16:32  foxworth
** Importing new setup sdk for open source project
**
** Revision 1.5  2005/07/29 19:43:29  nkinder
** 164436 - Updated copyrights, licensed code under LGPL, and licensed docs under OPL
**
** Revision 1.4  2005/07/27 01:04:50  nhosoi
** [161408] Open Sourcing Setup Util
** 1) Setup SDK -> Setup Util
** 2) Fixed doc to adjust to the current status (including branding -- debranding Netscape)
** 3) Eliminated to reference to unzip code and libwww (plus downloader, which uses libwww)
** 4) Added unzip_wrapper.pl which calls unzip command line utility (assuming installed on the system), which does the equivalent to the modified unzip code.
** 5) code cleanup
**
** Revision 1.3  2005/03/01 01:45:17  nhosoi
** Brand the Setup SDK as "Fedora"; updated copyrights
**
** Revision 1.2  2005/02/03 18:24:10  nhosoi
** rebranded setupsdk as brandx; updated the version number to 7.0
**
** Revision 1.1.1.1  2005/01/25 18:34:45  cvsadm
** Move SetupSDK from 6.2 development branch to TRUNK
**
** Revision 1.1.1.21.4.2  2002/04/13 00:59:11  djani
** Bug 533458: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.21.4.1  2002/04/13 00:08:55  djani
** Bug 540776: Backport from SetupsdkBranch to setupsdk_dev branch.
**
** Revision 1.1.1.21.2.2  2002/02/18 05:19:49  knguru
** Fix for bug 540776
** Code Reviewed by : Pete Rowley and Rich Megginson
** Also see : uninstaller.cc
** NOTE : Rich's suggestion to check for if the rmdir arg is a file is already incorporated(it was there in the existing code in the uninstall() function
**
** Revision 1.1.1.21.2.1  2002/02/06 15:58:31  srinidhi
**
** Bug Fix for 533458
** Reviewed by Miodrag
**
** Revision 1.1.1.21  2001/11/16 00:18:58  ahakim
** remove iplanet and airius refs
**
** Revision 1.1.1.20  2001/09/20 21:36:37  miodrag
** Change iPlanet to Netscape
**
** Revision 1.1.1.19  2001/07/03 18:45:09  svbld
** (mhein) Changes for x86 port
**
** Revision 1.1.1.18  2000/07/28 07:59:18  peterz
** fix bug related to directive KeepDirectories
**
** Revision 1.1.1.17  2000/07/19 04:16:47  peterz
** allocate more memory so when more than 10 products is being installed, it won't cause seg fault
**
** Revision 1.1.1.16  2000/03/06 01:01:32  peterz
** change "Netscape" to "iPlanet"
**
** Revision 1.1.1.15  1999/12/04 00:34:59  cxue
** SecurityCheck and RequireDomain support in setup.inf
**
** Revision 1.1.1.14  1999/11/12 01:58:00  cxue
** Keep going even no archive is defined in <product>.inf
**
** Revision 1.1.1.13  1999/10/13 00:19:26  cxue
** chnage <serverroot> and go forward
**
** Revision 1.1.1.12  1999/09/27 20:18:41  cxue
** check diskspace after selection is done
**
** Revision 1.1.1.11  1999/09/17 01:48:48  cxue
** fix long lasting cross-dependence bug
**
** Revision 1.1.1.10  1999/08/19 17:14:39  cxue
** compilation error
**
** Revision 1.1.1.9  1999/08/04 21:32:51  cxue
** Merge fix from setupsdk4.2 branch
**
** Revision 1.1.1.8  1999/07/29 00:23:56  svbld
** Merged the admin42_branch back onto the admin_branch..
**
** Revision 1.1.1.6.2.2  1999/07/26 21:08:40  shihcm
** bug#353225, (Msg)Error message missing which component need to install
**
** Revision 1.1.1.6.2.1  1999/06/14 20:46:56  cxue
** Get status for renamed file back from InfoZip
**
** Revision 1.1.1.6  1999/05/27 18:16:12  cxue
** Whoops, doesn't initialize s=NULL
**
** Revision 1.1.1.5  1999/05/25 18:04:07  cxue
** bug 353186, allow preInstall to customize archive files
**
** Revision 1.1.1.4  1999/05/20 19:01:02  cxue
** Patch database behaviors same as NT
**
** Revision 1.1.1.3  1999/04/21 22:43:13  cxue
** bug 349818. besides crc32, also check for filesize during upgrade installation
**
** Revision 1.1.2.40.2.19  1999/02/04 02:24:45  cxue
** remove temporary info file if process is killed by crtl-c. corect english sentences. Skip user/group dialog if needed
**
** Revision 1.1.2.40.2.18  1999/01/21 00:15:29  cxue
** Stop supporting downgrade installation
**
** Revision 1.1.2.40.2.17  1999/01/20 01:08:24  cxue
** support UpgradeOnly installation
**
** Revision 1.1.2.40.2.16  1998/12/18 19:37:09  cxue
** bad memory: reference memory which is released
**
** Revision 1.1.2.40.2.15  1998/12/17 00:45:51  cxue
** pass -r to postInstall
**
** Revision 1.1.2.40.2.14  1998/12/04 02:49:22  cxue
** billx uninstallation and customizable destination support
**
** Revision 1.1.2.40.2.13  1998/11/13 19:36:53  cxue
** version check doesn't work with multi-dependencies
**
** Revision 1.1.2.40.2.12  1998/11/12 18:35:47  cxue
** BillX:  hide all components for specific inf file (eg. base.inf)
**
** Revision 1.1.2.40.2.11  1998/11/04 19:42:34  pvo
** Enable option for skip build number checking
**
** Revision 1.1.2.40.2.10  1998/10/28 17:55:50  cxue
** support userdefined default setting
**
** Revision 1.1.2.40.2.9  1998/10/13 21:23:52  cxue
** compilation error
**
** Revision 1.1.2.40.2.8  1998/10/09 21:34:34  cxue
** compilation error
**
** Revision 1.1.2.40.2.6  1998/10/03 00:25:39  cxue
** move version related APIs to nsinst.cpp
**
** Revision 1.1.2.40.2.5  1998/09/30 23:22:16  cxue
** support multi archive files
**
** Revision 1.1.2.40.2.4  1998/09/25 23:08:39  cxue
** fileBegin to take extra argument crc32
**
** Revision 1.1.2.40.2.3  1998/09/25 03:57:34  pvo
** Relocation of Unzip
**
** Revision 1.1.2.40.2.2  1998/09/24 02:40:50  pvo
** Add buildNumber checking
**
** Revision 1.1.2.40.2.1  1998/09/24 01:57:40  cxue
** In the absence of components in current package, setup checks version number of dependency at server root
**
** Revision 1.1.2.40  1998/09/03 01:46:45  cxue
** Bug 316870. Make sure DS_ADMIN_DOMAIN only contans domain name (not dn)
**
** Revision 1.1.2.39  1998/08/22 00:44:05  cxue
** call insertLdifEntries after calling post-install
**
** Revision 1.1.2.38  1998/07/16 17:12:24  pvo
** 300883 - Fixed Express install problem. Reviewed by Tony
**
** Revision 1.1.2.37  1998/06/24 17:41:15  pvo
** 129804 - support UserDirectoryAuth
**
** Revision 1.1.2.36  1998/06/01 21:20:08  pvo
** 122446 - Uninstall prematurely deletes itself
**
** Revision 1.1.2.35  1998/05/01 01:06:51  pvo
** Fix problem with skipping extraction
**
** Revision 1.1.2.34  1998/04/21 19:58:44  pvo
** 106653 - create core components
**
** Revision 1.1.2.33  1998/04/14 01:05:19  pvo
** 113379 - add sighandler to restore screen mode
**
** Revision 1.1.2.32  1998/04/10 01:16:32  pvo
** Fix silent install problem w/ installType
**
** Revision 1.1.2.31  1998/04/06 21:17:58  pvo
** 112858 - Support for ForceExpressInstall flag
**
** Revision 1.1.2.30  1998/04/01 02:42:30  pvo
** 111799 - Interactive setup problem
**
** Revision 1.1.2.29  1998/03/24 22:29:45  pvo
** 110961 - Setup cannot handle long directory name.
**
** Revision 1.1.2.28  1998/03/19 04:17:46  pvo
** Support backing up files before extraction.
**
** Revision 1.1.2.27  1998/03/13 05:00:37  pvo
** 198489 - Change to reflect Registry vs. Account server concept
**
** Revision 1.1.2.26  1998/03/10 02:49:17  pvo
** Clean up compilation warnings.
**
** Revision 1.1.2.25  1998/03/09 22:09:37  pvo
** 108982 - Fixed selection problem with ctrl-B
**
** Revision 1.1.2.24  2000/03/03 21:39:58  pvo
** Fixed server seletion/dependency problem.
**
** Revision 1.1.2.23  1998/03/02 23:59:28  pvo
** 106912 - Pickup the right defaults in dialogs.
** 105320 - Improve logging of silent installation.
**
** Revision 1.1.2.22  1998/02/26 01:50:09  pvo
** Fixed Internal Error bug.
**
** Revision 1.1.2.21  1998/02/26 00:14:00  pvo
** 104962 - Installer hangs if running out of disk space
**
** Revision 1.1.2.20  1998/02/15 22:06:41  pvo
** 106679 - Support standalone installation of MCC.
**
** Revision 1.1.2.19  1998/02/04 00:01:54  pvo
** 104824 - DEC uninstall goes out of control
**
** Revision 1.1.2.18  1998/01/29 21:52:23  pvo
** Bug 105041 - add -h (help) screen for setup
** Bug 104968 - change default LDAP suffix to domain.com rather than example.com
** Others - removing redundant class variables
**
** Revision 1.1.2.17  1998/01/22 04:51:29  pvo
** Various changes:
**    - Added support for patching/update
**    - Changed selection dialogs to say Specify instead of Enter.
**
** Revision 1.1.2.16  1998/01/09 03:27:31  pvo
** Various code changes/additions:
**    - Write Components= to install.inf for silent install
**    - Relocate install.log to server root/setup directory
**    - Improve Component Selection screen to indicate number of
**      additional modules
**    - Fix problem with removing symbolic links
**
** Revision 1.1.2.15  1997/12/18 02:10:22  pvo
** Fixed problem on HPUX with tmpnam.
**
** Revision 1.1.2.14  1997/12/12 21:54:21  pvo
** Various bug fixes.
**
** Revision 1.1.2.13  1997/12/04 00:03:14  pvo
** Prepare support for DS installation.
**
** Revision 1.1.2.12  1997/11/26 02:03:01  pvo
** Restructure files and add Download support.
**
** Revision 1.1.2.11  1997/11/15 04:59:30  pvo
** Fixed memory leak problems and redo + operator for NSString.
**
** Revision 1.1.2.10  1997/11/14 19:23:13  pvo
** Fixed memory leaks.
**
** Revision 1.1.2.9  1997/11/14 09:11:32  pvo
** Various changes to strengthen code.
**
** Revision 1.1.2.8  1997/11/13 19:28:41  pvo
** More new stuffs. Changed .pi to .inf.
**
** Revision 1.1.2.7  1997/11/12 02:09:33  pvo
** I have no idea. Just additional code and bug fixes.
**
** Revision 1.1.2.6  1997/11/11 23:17:35  pvo
** Implemented preUninstall and postUninstall.
**
** Revision 1.1.2.5  1997/11/11 05:41:10  pvo
** Added dependency checking for uninstallation.
**
** Revision 1.1.2.4  1997/11/10 22:20:32  pvo
** Fixed dependencyCheck.
**
** Revision 1.1.2.3  1997/11/09 19:30:12  pvo
** More code for uninstallation. Changed ns-remove.cc to uninstall.cc.
** Removed second password prompt when asking Ldap user.
**
** Revision 1.1.2.2  1997/11/08 18:10:16  pvo
** Added fileBegin.
**
** Revision 1.1.2.1  1997/11/08 16:47:48  pvo
** New. Uninstallation support.
**
**
*********************************************************************/

/*********************************************************************
**
** Includes
**********************************************************************
*/
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
extern "C"
{
/* #include "../lib/unzip/unzip.h" */
}

#include "dialog.h"
#include "uninstall.h"
#include "ux-setup.h"
#include "product.h"
#include "ldapu.h"
#include "setupinst.h"

const char *EXTRACT_PROGRAM = "/bin/tar";
const char *EXTRACT_FLAGS = "xof";

int fileBegin(const char *fileName, void *product, unsigned long crc32, long size);
void fileEnd(const char *fileName, void *product, unsigned long crc32, int status);
void unzipEnd(unsigned long size, void *product);
void sigHandler(int sigNo);
void (*unzipSigHandler)(int) = NULL;

/*
 * Check to see whether rename happens during unzip. The purpose is to popup 
 * warning messages after all package are unziped.  
 */
Bool unzipRename = False;

Bool Product::_needSecurity = True;
int deleteEmptyDirRecursively(const char* input)
{
   DIR *dirp;
   struct dirent *dir;
   int ret = 0;

    //Open the directory to read its contents
   dirp = opendir(input);

   if (dirp)
   {
        //Read the contents of the directory
       while ((dir = readdir(dirp)) != NULL)
       {
            //Skip the . and .. entries
          if (strcmp(dir->d_name, ".") != 0 &&
              strcmp(dir->d_name, "..") != 0)
          {
                char szdir[2048];
                //Create an absolute path
                sprintf(szdir,"%s/%s",input, dir->d_name);
                //Try removing the directory
             int i = rmdir(szdir);
                if (i != 0)
                {
                    // Means that the directory is not empty
                    /* 
                    Call this function recursively so that if the directory has
                    subdirs then try deleting the subdirs in the hope that they are empty
                    */
                ret = deleteEmptyDirRecursively(szdir);
                }
          }
       }
       closedir(dirp);
    }
    //Now that the iteration is over, try removing this directory
   ret = rmdir(input);
   return ret;
}
////////////////////////////////////////////////////////////////////////////
// CLASS : Product
////////////////////////////////////////////////////////////////////////////

void
Product::init()
{
   clean();

   _packageInfo = NULL;
   _selected = False;
   _skipped = False;
   _skippedConfig = False;
   _exists = False;
   _compatible = True;
   _securityChecked = True;
   _downgrade = False;
   _installed = False;
   _components = new Components; // Create empty component list
   _logFile = NULL;
   _archiveSize = 0;
}

Product::Product ()
   :_packageInfo(0), _components(0), _logFile(0)
{
   init();
}

Product::Product (PackageInfo *package)
   :_packageInfo(0), _components(0), _logFile(0)
{
   init();
   _packageInfo = package;
}

Product::~Product()
{
   clean();
}

void
Product::clean()
{
/* PVO - !!!!
   if (_packageInfo) 
   { 
      delete _packageInfo;
   }
*/
   _packageInfo = NULL;

   if (_components) 
   {
      delete _components; 
      _components = NULL;
   }
   if (_logFile)
   {
      delete _logFile;
      _logFile = NULL;
   } 
}

PackageType
Product::packageType() const
{
   PackageType type = Normal;
   const char *pkgType = get(PKG_TYPE);
   if (pkgType != NULL)
   {
      if (!strcasecmp(pkgType, "Patch"))
      {
         type = Patch;
      }
      else if (!strcasecmp(pkgType, "Update"))
      {
         type = Update;
      }
   }
   return type;
}
      
Bool
Product::isChecked() const
{
   const char *checked = _packageInfo->get(PKG_CHECKED);
   if (checked == NULL || strcasecmp(checked, "True"))
   {
      return False;
   }
   else
   {
      return True;
   }
}

Bool
Product::isMandatory() const
{
   const char *mandatory = _packageInfo->get(PKG_MANDATORY);
   if (mandatory == NULL || strcasecmp(mandatory, "True"))
   {
      return False;
   }
   else
   {
      return True;
   }
}

Bool
Product::isVisible() const
{
  const char *visible = _packageInfo->get(PKG_VISIBLE);

  if (visible == NULL || !strcasecmp(visible, "True"))
  {
    if (numComponents() == 0)
      return True;

    for (int i = 0; i < numComponents(); i++)
    {
      Product *p;
      p =  getComponent(i);
      if (p->isVisible() == True && p->skipped() == False)
      {
          return True;
      }
    }
    return False;
  }
  else
  {
     return False;
  }
}

void
Product::selectDefaults()
{
   if (isChecked() || isMandatory() || numComponents())
   {
      select();
      for (int i = 0; i < numComponents(); i++)
      {
         getComponent(i)->selectDefaults();
      }
   }
}
 
void
Product::selectAll()
{
   // Select myself first
   select();
   for (int i = 0; i < numComponents(); i++)
   {
      getComponent(i)->selectAll();
   }
}
   
void
Product::deSelectAll()
{
   // Select myself first
   deSelect();
   for (int i = 0; i < numComponents(); i++) 
   {
      getComponent(i)->deSelectAll(); 
   }
}
   
int
Product::retrieveSuiteSpot(const NSString & sourceDir, const char *fileName)
{
   PackageInfo *sp = NULL;
   char tstr[BIG_BUF];
   Product *prd = NULL;
   const char *infoFile;
   char **componentList = NULL;

   // This is top level master configuration file
   init();

   sprintf(tstr, "%s/%s", sourceDir.data(), fileName);
   _packageInfo = new PackageInfo(tstr);

   if (_packageInfo->isEmpty() || (componentList = _packageInfo->getListItems(PKG_COMPONENTS)) == NULL || _packageInfo->numSections() == 0)
   {
      return -1;
   }

   for (int i = 0; componentList[i] != NULL; i++)
   {
      sp = _packageInfo->getSection(componentList[i]);

      if (sp == NULL)
      {
         continue;
      }

      if ((infoFile = sp->get("ComponentInfoFile")) != NULL)
      {
         prd = new Product();

         if (prd->retrieveProduct(sourceDir, infoFile, componentList[i]) == 0)
         {
            _components->addComponent(prd);
         }
         else
         {
            delete prd;
            prd = NULL;
         }
      }
   }

   if (componentList)
   {
      _packageInfo->freeListItems(componentList);
   }

   if (numComponents() == 0)
   {
      return -1;
   }
   /*
    * Indicate that this is the "top" components
    */
   _components->top();

   return 0;
}

int
Product::retrieveProduct(const NSString & sourceDir, const char *fileName, const char *nickName)
{
   PackageInfo *sp = NULL;
   char tstr[BIG_BUF];
   Product *prd = NULL;
   int i;
   NSString logFile;
   NSString infFile;
   NSString setupDir;
   char **componentList = NULL;

   init();

   sprintf(tstr, "%s/%s", sourceDir.data(), fileName);
   _packageInfo = new PackageInfo(tstr);

   if (_packageInfo->isEmpty())
   {
      return -1;
   }

   if (_packageInfo->get(PKG_NICKNAME) == NULL)
   {
      _packageInfo->set(PKG_NICKNAME, nickName);
   }

   setupDir = _packageInfo->get(PKG_NICKNAME);

   infFile  = setupDir + "/" + _packageInfo->get(PKG_NICKNAME) + INF_SUFFIX;

   logFile = setupDir + "/" + _packageInfo->get(PKG_NICKNAME) + LOG_SUFFIX;
   _packageInfo->set("SetupDir", setupDir);
   _packageInfo->set("InfFile", infFile);

    // default setting for children to inherit

    const char * visible   = _packageInfo->get(SETUP_INF_VISIBLE);
    const char * mandatory = _packageInfo->get(SETUP_INF_MANDATORY);
    const char * checked   = _packageInfo->get(SETUP_INF_CHECK);


   if (_packageInfo->numSections() == 0)
   {
      // Not modularized product
      if (_packageInfo->get(PKG_ARCHIVE) == NULL)
      {
        setupLogMessage(WARN, "Setup", "Archive is not defined in %s", _packageInfo->get(PKG_NAME));
        //  return -1;
      }
      _packageInfo->set("LogFile", logFile);
   }
   else
   {
      componentList = _packageInfo->getListItems(PKG_COMPONENTS);
      if (componentList == NULL)
      {
         return -1;
      }

      for (i = 0; componentList[i] != NULL; i++)
      {
         sp = _packageInfo->getSection(componentList[i]);

         if (sp == NULL)
         {
            continue;
         }

         // set default value from [general] section of this inf file
         if (visible && sp->get(SETUP_INF_VISIBLE) == NULL)
            sp->set(SETUP_INF_VISIBLE, visible);
         if (mandatory && sp->get(SETUP_INF_MANDATORY) == NULL)
            sp->set(SETUP_INF_MANDATORY, mandatory);
         if (checked && sp->get(SETUP_INF_CHECK) == NULL)
            sp->set(SETUP_INF_CHECK, checked);

         logFile = setupDir + "/" + sp->get(PKG_NICKNAME) + ".log";

         sp->set("SetupDir", setupDir);
         sp->set("LogFile", logFile);
         sp->set("InfFile", infFile);         


         prd = new Product(sp);
         _components->addComponent(prd);
      }
   }

   if (componentList)
   {
      _packageInfo->freeListItems(componentList);
   }


   return 0;
}
int
Product::retrieveDownloadProduct(NVPair *downloadInfo)
{
   PackageInfo *sp = NULL;
   Product *prd = NULL;
   char **componentList = NULL;
   int i, j;

   init();

   _packageInfo = downloadInfo;

   if (_packageInfo->isEmpty() || _packageInfo->numSections() == 0 ||
        (componentList = _packageInfo->getListItems("Packages")) == NULL)
   {
      // Wrong package info format
      return -1;
   }

   // Now find all the packages that could be downloaded
   for (i = 0; componentList[i] != NULL; i++)
   {
      sp = _packageInfo->getSection(componentList[i]);
      if (sp == NULL)
      {
         continue;
      }
      prd = new Product(sp);
      _components->addComponent(prd);
   }
   _packageInfo->freeListItems(componentList);

   // and find all the associated components
   for (i = 0; i < numComponents(); i++)
   {
      prd = getComponent(i);
      sp = prd->package();
      componentList = sp->getListItems(PKG_COMPONENTS);
      if (componentList != NULL)
      {
         for (j = 0; componentList[j] != NULL; j++)
         {
            PackageInfo *spp = _packageInfo->getSection(componentList[j]);
            Product *sprd;

            if (spp == NULL)
            {
               continue;
            }
            sprd = new Product(spp);
            prd->_components->addComponent(sprd);
         }
         sp->freeListItems(componentList);
      }
   }

   _components->top();

   return 0;
}

#if 0
int
Product::download(Inet &inet, const char *source, const char *destination)
{
   NSString sourceURL = source;
   NSString targetDir = destination;
   NSString fromFile, toFile;
   const char *src, *dest;
   int err;
   int numFiles = 0;
   const char *file;
   char fileNum[XSM_BUF];

   file = _packageInfo->get("Files");
   if (file == NULL || (numFiles = atoi(file)) == 0)
   {
      return 0;
   }

   src = _packageInfo->get("Location");
   if (src)
   {
      sourceURL = sourceURL + "/" + src;
   }

   dest = _packageInfo->get("Destination");

   if (dest)
   {
      targetDir = targetDir + "/" + dest;
   }

   err = InstUtil::createDirectory(targetDir, 0755);

   if (err)
   {
      return err; // PVO do something
   }

   for (int i = 1; i <= numFiles; i++)
   {
      sprintf(fileNum, "File%d", i);
      file = _packageInfo->get(fileNum);
      if (file == NULL)
      {
         continue;
      }
      fromFile = sourceURL + "/" + file;
      toFile   = targetDir + "/" + file;
      err = inet.retrieveURL(fromFile, toFile);
      if ( !err)
      {
         chmod(toFile.data(), S_IRWXU | S_IRWXG);
      }
      else
      {
         err = -1; // PVO - continue, probably print an error
      }
   }
   return err;
}
#endif

int
Product::numComponents() const
{
   return _components->numComponents();
}

int
Product::numVisibleComponents() const
{
   int count = 0;

   for (int i = 0; i < _components->numComponents(); i++)
   {
      if (_components->getComponent(i)->isVisible() && 
          !_components->getComponent(i)->skipped())
      {
         ++count;
      }
   }
   return count;
}

Product *
Product::getComponent(int i) const
{
   return _components->getComponent(i);
}

Product *
Product::getComponent(const char *nickName) const
{
   return _components->getComponent(nickName);
}

const char *
Product::get(const char *attr) const
{
   return _packageInfo->get(attr);
}

void 
Product::set(const char *attr, const char *value)
{
   _packageInfo->set(attr, value);
}

NVPair *
Product::dependencies(const char *field) const
{
   const char *item;
   int i = 0;
   char tstr[XSM_BUF];

   if (_packageInfo == NULL)
   {
      return NULL;
   }

   NVPair *p = new NVPair();
   while ((item = _packageInfo->getListItem(field, i)) != NULL)
   {
      char *s = strchr((char *)item,'/');
      if (s != NULL)
      {
         strncpy(tstr, item, s - item);
         tstr[s-item] = 0;
         p->set (tstr, s+1);
      }
      else
      {
         p->set (item, "4.0");
      }
      i++;
   }
   if (p->isEmpty())
   {
      delete p;
      return NULL;
   }
   else
   {
      return p;
   }
}

void
Product::getAllComponents(Components & components)
{
   int numComponents = this->numComponents();

   // First, add myself to the component list if its OK
   if (numComponents == 0)
   {
      components.addComponent(this);
   }

   for (int i = 0; i < numComponents; i++)
   {
      Product *p = this->getComponent(i);
      p->getAllComponents(components);
   }

   return;
}
   
int
Product::checkSecurity(const Product *other) const
{
   const char *mySecurity, *otherSecurity;
   int status;

   if (_needSecurity == False)
     return 1;

   mySecurity = this->get(PKG_SECURITY);
   otherSecurity = other->get(PKG_SECURITY);

   status = setupSecurityCompare(mySecurity, otherSecurity);

   return status;
}

int
Product::checkCompatibility(const Product *other, Bool checkBuildNum) const
{
   const char *myVersion, *otherVersion;
   const char *myCompatible, *otherCompatible;
   const char *myBuildNum, *otherBuildNum;
   int rc;

   myVersion = this->get(PKG_VERSION);
   otherVersion = other->get(PKG_VERSION);
   myCompatible = this->get(PKG_COMPATIBLE);
   otherCompatible = other->get(PKG_COMPATIBLE);
   myBuildNum = this->get(PKG_BUILDNUMBER);
   otherBuildNum = other->get(PKG_BUILDNUMBER);
   
   // I am compatible with the other package if
   // My version and the other's version is the same or
   // I am older and the other compatible version <= my version or
   // I am newer and my compatible version <= the other version
   //   or

   rc = setupVersionCompare (myVersion, otherVersion);
   if (rc == 0)
   {
      if (checkBuildNum)
      {
         rc = setupBuildCompare(myBuildNum, otherBuildNum);
      }
   }
   else if (rc > 0)
   {
      // I am newer
      if (setupVersionCompare(myCompatible, otherVersion) <= 0)
      {
         // OKAY, compatible
         rc = 1;
      }
      else
      {
         rc = -1;
      }
   }
   else 
   {
      rc = -2;
   }
   return rc; 
}

Bool
Product::isIdentical(const Product *other) const
{

   if (setupVersionCompare(this->get(PKG_VERSION), other->get(PKG_VERSION)) == 0
              &&
           checkSecurity(other) == 0
              &&
           setupBuildCompare(this->get(PKG_BUILDNUMBER), other->get(PKG_BUILDNUMBER)) == 0
              &&
           setupRevisionCompare(this->get(PKG_REVISION), other->get(PKG_REVISION)) == 0
           )
       return True;
   else
       return False;
}



NSString
Product::extract( const char *destination, const char *archive, 
                  const char *program, const char *whichFile,
                  NVPair *installInfo)
{
   fflush(stdout);
   fflush(stderr);

   if (program)
   {
      return untar (destination, archive, program, EXTRACT_FLAGS, whichFile);
   }
   else
   {
      return unzip (destination, archive, whichFile, installInfo);
   }
}

NSString
Product::untar(const char *dest,
               const char *archive,
               const char *program,
               const char *arg,
               const char *whichFile)
{
   NSString command;
   NSString errMsg;
   int ret = 0;

   command = command + program + " " + EXTRACT_FLAGS + " " + archive;

   if (whichFile)
   {
      command = command + " " + whichFile;
   }
   
   ret = InstUtil::execProgram(command, dest);

   if (ret)
   {
      errMsg = "Error untarring archive";
   }

   return errMsg;

}

NSString
Product::unzip(const char *dest,
               const char *zipFile,
               const char *whichFile,
               NVPair *installInfo)
{
   NSString logFileName;
   NSString tmpfilename;
   int rc;
   NSString errMsg;
   const char *serverRoot = installInfo->get(SERVER_ROOT);

   if ((NSString)NULL == _logDest)
   {
      _logDest = NSString(serverRoot) + "/" + SETUP_DIR;
   }
   logFileName = _logDest + "/" + get("LogFile");

   if (!InstUtil::dirExists(InstUtil::dirName(logFileName)))
   {
       InstUtil::createDirectory(InstUtil::dirName(logFileName), 0755);
   }
   tmpfilename = InstUtil::dirName(logFileName) + "/size.tmp";
   if (NULL == _logFile)
   {
      _logFile = new NVPair (logFileName);
   }

   char program[1024];

   if (InstUtil::fileExists(logFileName))
   {
      snprintf(program, sizeof(program),
               "perl unzip_wrapper.pl -x -z %s -d %s", zipFile, dest);
      if (memchr(program, 0, sizeof(program)))
      {
         rc = InstUtil::execProgram(program);
      }
      else
      {
         errMsg = NSString("Invalid program: ") + NSString(program);
      }
   }
   else
   {
      snprintf(program, sizeof(program),
            "perl unzip_wrapper.pl -x -z %s -d %s -i -l %s -r %s -s %s",
            zipFile, dest, logFileName.data(), serverRoot, tmpfilename.data());
      if (memchr(program, 0, sizeof(program)))
      {
         FILE *fp;
         rc = InstUtil::execProgram(program);
         if (fp = fopen(tmpfilename.data(), "r"))
         {
            long size;
            if (EOF != fscanf(fp, "%d", &size))
            {
               long newSize = getArchiveSize() + (size / 1024 + 1);
               setArchiveSize(newSize);
            }
            fclose(fp);
            unlink(tmpfilename.data());
         }
      }
      else
      {
         errMsg = NSString("Invalid program: ") + NSString(program);
      }
   }

   // Restore signal handlers to defaults
   signal (SIGINT, SIG_DFL);

#ifdef SIGBUS
   signal(SIGBUS, SIG_DFL);
#endif /* SIGBUS */

#ifdef SIGSEGV
   signal(SIGSEGV, SIG_DFL);
#endif /* SIGSEGV */

   return errMsg;
      
}


void 
Product::unzipInfo(const char *zipFile, NVPair *installInfo)
{
   NSString logFileName;
   NSString tmpfilename;
   int rc;
   const char *serverRoot = installInfo->get(SERVER_ROOT);

   char program[1024];

   if ((NSString)NULL == _logDest) {
      _logDest = NSString(serverRoot) + "/" + SETUP_DIR;
   }

   logFileName = _logDest + "/" + get("LogFile");
   if (!InstUtil::dirExists(InstUtil::dirName(logFileName)))
   {
       InstUtil::createDirectory(InstUtil::dirName(logFileName), 0755);
   }
   tmpfilename = InstUtil::dirName(logFileName) + "/size.tmp";
   if (NULL == _logFile)
   {
      _logFile = new NVPair (logFileName);
   }

   if (!InstUtil::fileExists(logFileName))
   {
      snprintf(program, sizeof(program),
               "perl unzip_wrapper.pl -z %s -i -l %s -r %s -s %s",
               zipFile, logFileName.data(), serverRoot, tmpfilename.data());
      if (memchr(program, 0, sizeof(program)))
      {
         FILE *fp;
         rc = InstUtil::execProgram(program);
         if (fp = fopen(tmpfilename.data(), "r"))
         {
            long size;
            if (EOF != fscanf(fp, "%d", &size))
            {
               long newSize = getArchiveSize() + (size / 1024 + 1);
               setArchiveSize(newSize);
            }
            fclose(fp);
            unlink(tmpfilename.data());
         }
      }
   }
}

void setUnzipSigHandler(void (*handler)(int))
{
   unzipSigHandler = handler;
}

void messagePrn()
{
}

void
Product::updateInstalledInfo(NVPair *info, const NSString &bindDN, const NSString &bindPwd)
{
   char prodinfo[MED_BUF];
   FILE *nyr;
   time_t t = time(NULL);
   const char *sroot = info->get(SERVER_ROOT);
   PackageInfo *p = _packageInfo;

   sprintf(prodinfo, "%s/nyr", sroot);
   if (nyr = fopen(prodinfo,"w"))
      fclose(nyr);

   // Create new prodinfo
   sprintf(prodinfo, "%s/bin/%s/prodinfo", sroot, get(PKG_NICKNAME));
   p->stampTime();
   p->write(prodinfo);

   return;
}

int
Product::preInstall(const NSString & serverRoot, const NSString & sourceDir, 
             const char *installType, 
             InstallMode installMode, const char *logFile,
             const char *infoFile) const
{
   int err = 0;
   NSString program;
   NSString source;
   const char *s;

   if (installType == NULL)
   {
      installType = TYPICAL_TYPE;
   }

   if (!strcasecmp(installType, TYPICAL_TYPE))
   {
      s = get(PKG_FORCE_XPRESS);
      if (s && !strcasecmp(s, "True"))
      {
         installType = EXPRESS_TYPE;
      }
   }

   s = get(PKG_PREINSTALL);
   if (s != NULL && *s)
   {
      source = sourceDir + "/" + get("SourcePath");
      program = NSString ("./") + s;

      if (installMode != Silent)
      {
         program = program + " -m " + installType; // install type
      }
      else
      {
         program = program + " -s -l " + logFile;
      }

      program = program + " -f " + infoFile;

      if (this->installed() || this->exists())
      {
         program = program + " -r";
      }
   
      err = InstUtil::execProgram(program,source);
   }

   return err;
}

int
Product::postInstall(const NSString & serverRoot, const NSString & sourceDir, const char *infoFile, const InstallInfo *installInfo) const
{
   NSString program;
   NSString source;
   const char *s;
   int err = 0;

   s = get(PKG_POSTINSTALL);

   if (s != NULL && *s)
   {
      if (s[0] == '/')
      {
         program = s;
      }
      else
      {
         program = serverRoot + "/" + s;
      }

      source = serverRoot;
      program = program + " -f " + infoFile;

      if (this->installed() || this->exists())
      {
         program = program + " -r";
      }

      err = InstUtil::execProgram(program, source);
   }

   if (err == OKAY)
   {
      err = insertSchemaFiles(sourceDir, installInfo);
      err = insertLdifFiles(sourceDir, installInfo);
   }

   return err;
}


int
Product::insertSchemaFiles(const NSString & sourceDir, const InstallInfo *installInfo) const
{
   NSString sourceSchemaDir;
   const char *s;
   int err = 0;
   const char *schemaFiles;
    NSString filename;

   schemaFiles = get(SETUP_INF_UPDATE_SCHEMA);
   if (schemaFiles != NULL)
   {
      char *buf = NULL;
      char *tok;
      const char *ldapURL;
      const char *ldapUser;
      const char *ldapPwd;

      sourceSchemaDir = sourceDir + "/" + get("SourcePath") + "/";

      ldapURL = installInfo->get(CONFIG_LDAP_URL);
      ldapUser = installInfo->get(MC_ADMIN_ID);
      ldapPwd = installInfo->get(MC_ADMIN_PWD);

      buf = strdup(schemaFiles);

      for (tok = strtok(buf,", "); tok; tok = strtok(NULL,", "))
      {
         filename = sourceSchemaDir + tok;
         err = setupInsertPluginSchemaEntries(ldapURL, ldapUser, ldapPwd, filename);
      }
      if (buf)  free(buf);
   }

   return err;
}


int
Product::insertLdifFiles(const NSString & sourceDir, const InstallInfo *installInfo) const
{
   NSString sourceLdifDir;
   const char *s;
   int err = 0;
   const char *ldifFiles;
    NSString ldifPath;

   ldifFiles = get(PKG_UPDATE_LDIF);
   if (ldifFiles != NULL)
   {
      /*
       * Insert ldif files to DS
       */
      char *buf = NULL;
      char *tok;
      const char *ldapURL;
      const char *ldapUser;
      const char *ldapPwd;
      char *adminDomainDN = NULL;
      const char *s;

      sourceLdifDir = sourceDir + "/" + get("SourcePath") + "/";

      ldapURL = installInfo->get(CONFIG_LDAP_URL);
      ldapUser = installInfo->get(MC_ADMIN_ID);
      ldapPwd = installInfo->get(MC_ADMIN_PWD);

      if (installInfo->get(DS_ADMIN_DOMAIN) != NULL)       
      {
         adminDomainDN = formAdminDomainDN(installInfo->get(DS_ADMIN_DOMAIN));
      }

      buf = strdup(ldifFiles);

      for (tok = strtok(buf,", "); tok; tok = strtok(NULL,", "))
      {
         ldifPath = sourceLdifDir + tok;
         err = insertPluginLdifEntries(ldapURL, adminDomainDN, ldapUser, ldapPwd, ldifPath);
         if (err != OKAY)
            break;
      }
      if (buf)  free(buf);
      if (adminDomainDN) free(adminDomainDN);
   }

   return err;
}



/*
 *  Not like get(), getInfo() looks for name defined in installation cache
 *  first before looking inside product section.
 *  This allows product's preInstall to overwrite name/value defined inside product section
 *  The purpose to use this instead of get() for future release.
 */

const char * Product::getInfo(NVPair *installInfo, const char *name)
{
   const char *s = NULL;
   NVPair *section;

   if (section = installInfo->getSection(get(PKG_NICKNAME)))
      s = section->get(name);

   if (s == NULL)
      s = get(name);

   return s;
}

NSString
Product::install(const NSString & serverRoot, const NSString & sourceDir, NVPair *installInfo)
{
   NSString installedDir;
   NSString errMsg;
   const char *s;
   char *buffer;
   char *archive;
   NSString filePath;
   NSString productName;
   const char *dest;
   NVPair *section;

   s = getInfo(installInfo, "Destination");

   if (s == NULL)
   {
      installedDir = serverRoot;
   }
   else
   {
      installedDir = s;
   }

   if ((NSString)NULL == _logDest)
      _logDest = serverRoot + "/" + SETUP_DIR;

   /*
    * Before extracting, backup files
    */
   backupFiles(PKG_BACKUPFILES, installedDir, serverRoot);
   backupFiles(PKG_RESTOREFILES, installedDir, serverRoot);

   buffer = strdup(getInfo(installInfo, PKG_ARCHIVE));

   archive = strtok(buffer, ", ");
   while (archive != (char *) NULL)
   {
      filePath = sourceDir + "/" + get("SourcePath") + "/" + archive;
      errMsg = extract(installedDir, filePath, NULL, NULL, installInfo);
      if (errMsg != (char *) NULL)
      {
         break;
      }
      archive = strtok(NULL, ", ");
   }

   free(buffer);
   return errMsg;
}

int
Product::unInstall(const NSString & serverRoot) const
{
   /*
    * Get KeepFiles, KeepDir, logFile, infFile
    */
   char **keepFiles = _packageInfo->getListItems(PKG_KEEPFILES);
   char **keepDir = _packageInfo->getListItems(PKG_KEEPDIRS);
   NSString logFile;
   char buf[MED_BUF];
   const char *s;
   FILE *fp;
   struct stat fi;
   NVPair dirList;
   NSString postProg;

   // Before doing uninstall, move the postRemove file to
   // server root and change it name
   postProg = _packageInfo->get(PKG_POSTUNINSTALL);
   if (postProg != (char *) NULL && postProg.data()[0] != 0)
   {
      relocate(serverRoot, postProg);
      _packageInfo->set(PKG_POSTUNINSTALL, postProg);
   }

   logFile = _packageInfo->get("LogFile");
   if (logFile != (char *) NULL)
   {
      logFile = serverRoot + "/" + SETUP_DIR + "/" + logFile;

      if (fp = fopen(logFile.data(), "r"))
      {
         while (fscanf(fp, "%s", buf) != EOF)
         {
            char suffix[MED_BUF] = "\0";
            char *p;

            s = buf + serverRoot.length() + 1;

            p = strrchr((char *)s, '/');

            if(p != NULL)
            {
                strncpy(suffix, s, p - s + 1);
            }
            else
            {
                strcpy(suffix, s);
            } 


            if ((_packageInfo->inList(PKG_KEEPFILES, s) == True))
            {
                continue;
            }
            else if (_packageInfo->inList(PKG_KEEPDIRS, suffix) == True)
            {
                continue;
            }
            else
            {
                if (lstat(buf, &fi) == 0)
                {
                    if (S_ISDIR(fi.st_mode))
                    {
                        if (_packageInfo->inList(PKG_KEEPDIRS, s) == True)
                        {
                            continue;
                        }
                        else
                        {
                            dirList.add("Dir", buf);
                        }
                    }
                    else
                    {
                        unlink(buf);
                    }
                }
            }
         }

         fclose (fp);

         unlink(logFile.data());

         // Now remove the directories
         for (int i = dirList.numListItems("Dir")-1;  i >= 0 ; i--)
         {
            s = dirList.getListItem("Dir", i);
            if (s != NULL)
            {
               deleteEmptyDirRecursively(s);
            }
         }
      }
   }

   if (keepFiles) 
   { 
      _packageInfo->freeListItems (keepFiles);
   }

   if (keepDir) 
   {
     _packageInfo->freeListItems (keepDir);
   }

   return 0;
}

int
Product::preUnInstall(const NSString & serverRoot, const char *infoFile) const
{
   const char *s;
   NSString program;
   int err = 0;

   s = get(PKG_PREUNINSTALL);
   if (s != NULL && *s)
   {
      if (s[0] == '/')
      {
         program = s;
      }
      else
      {
         program = serverRoot + "/" +s;
      }

      program = program + " -f " + infoFile;

      err = InstUtil::execProgram(program);
   }

   return  err;
}

int
Product::postUnInstall(const NSString & serverRoot, const char *infoFile) const
{
   const char *s;
   NSString program;
   int err;

   s = get(PKG_POSTUNINSTALL);
   if (s != NULL && *s)
   {
      if (s[0] == '/')
      {
         program = s;
      }
      else
      {
         program = serverRoot + "/" +s;
      }

      program = program + " -f " + infoFile;

      err = InstUtil::execProgram(program);
   }

   return  err;
}

void
Product::backupFiles(const char *backupType, const NSString &installedDir, const NSString & serverRoot)
{
   NSString backupSource;
   NSString backupTarget;
   char buf[SML_BUF];
   char **backupList, **files;

   files = backupList = package()->getListItems(backupType);
   if (files)
   {
      while (*files)
      {
         backupSource = *files;

         if (backupSource.data()[0] != '/')
         {
            backupSource = installedDir + "/" + backupSource;
         }

         if (!strcasecmp(backupType, PKG_BACKUPFILES))
         {
            get_gmtime(buf, SML_BUF);

            buf[strlen(buf)-1] = 0;

         }
         else
         {
            strcpy(buf, "backup");
         }

         backupTarget = backupSource + "." + buf;

         if (InstUtil::copyFile (backupTarget, backupSource) == 0)
         {
            char *pkg, *module;
            pkg = strdup(get("LogFile"));
            if (pkg)
            {
               module = strchr(pkg, '/');
               if (module)
               {
                  *module = 0;
                  ++module;
                  *(strchr(module,'.')) = 0;
               }
               else
               {
                  module = pkg;
               }

               if (!strcasecmp(backupType, PKG_BACKUPFILES))
               {
                  logUninstallInfo(serverRoot.data(),
                     pkg,
                     module,
                     backupTarget.data());
               }
               free(pkg);
            }
         }
         ++files;
      }
   }

   if (backupList)
   {
      NVPair::freeListItems(backupList);
   }
}

void
Product::restoreFiles (const NSString &installedDir)
{
   NSString restoreSource;
   NSString restoreTarget;
   char buf[SML_BUF];
   char **restoreList, **files;

   files = restoreList = package()->getListItems(PKG_RESTOREFILES);
   if (files)
   {
      while (*files)
      {
         restoreTarget = *files;

         if (restoreTarget.data()[0] != '/')
         {
            restoreTarget = installedDir + "/" + restoreTarget;
         }

         restoreSource = restoreTarget + "." + "backup";

         if (InstUtil::copyFile (restoreTarget, restoreSource) == 0)
         {
            unlink(restoreSource.data());
         }
         ++files;
      }
   }

   if (restoreList)
   {
      NVPair::freeListItems(restoreList);
   }
}


////////////////////////////////////////////////////////////////////////////
// CLASS: Components
////////////////////////////////////////////////////////////////////////////

Components::Components()
   :_numComponents(0),_products(0), _top(False)
{
}

Components::~Components()
{
   if (_products) 
   {
      for (int i = 0; i < _numComponents; i++)
      {
         delete _products[i];
      }
      free(_products);
   }
}

void
Components::selectComponent(const char *nickName)
{
   int i;
   if (nickName)
   {
      for (i = 0; i < _numComponents; i++)
      {
         if (!strcasecmp(nickName, _products[i]->get(PKG_NICKNAME)) && _products[i]->compatible() && !_products[i]->downgrade() && !_products[i]->installed())
         {
            _products[i]->select();
         }
      }
   }
}

void
Components::deselectComponent(const char *nickName)
{
   int i;
   if (nickName)
   {
      for (i = 0; i < _numComponents; i++) 
      { 
         if (!strcasecmp(nickName, _products[i]->get(PKG_NICKNAME)))
         { 
            _products[i]->deSelect();
         }
      }
   }
}

void
Components::addComponent(Product *product)
{
   if (_numComponents == 0)
   {
      _products = (Product **) malloc(101 * sizeof (Product *));
   }
/*
   else
   {
      _products = (Product **) realloc(_products, sizeof(Product *) * (_numComponents+1));
   }
*/
      
   _products[_numComponents++] = product;
   _products[_numComponents] = NULL;
}

Product *
Components::getComponent(const char *nickName)
{
   if (nickName != NULL)
   {
      for (int i = 0; i < _numComponents; i++) 
      { 
         if (!strcasecmp(nickName, _products[i]->get(PKG_NICKNAME)))
         { 
            return _products[i];
         }
      }
   }
   return NULL;
}

Product *
Components::getComponent (int i)
{
   return _products[i];
}

int
Components::numSelectedComponents() const
{
   register int i;
   int ret = 0;

   for (i = 0; i < _numComponents; i++) 
   {
      if (_products[i] && _products[i]->isSelected())
      {
         ret++;
      }
   }
   return ret;
}

Bool
Components::useLdap()
{
   register int i;
   Bool ret = False;

   for (i = 0 ; i < _numComponents; i++)
   {
      if (_products[i] && _products[i]->isSelected())
      {
         const char *useLdap = _products[i]->get(PKG_USELDAP);
         if (useLdap && !strcasecmp(useLdap, "True"))
         {
             ret = True;
             break;
         }
      }
   }
   return ret;
}

Bool
Components::userDirAuth()
{
   register int i;
   Bool ret = False;

   for (i = 0 ; i < _numComponents; i++)
   {
      if (_products[i] && _products[i]->isSelected())
      {
         const char *useLdap = _products[i]->get(PKG_USERDIR_AUTH);
         if (useLdap != NULL && !strcasecmp(useLdap, "True"))
         {
             ret = True;
             break;
         }
      }
   }
   return ret;
}

Bool
Components::hasVisible()
{
   register int i;
   Bool ret = False;

   for (i = 0; i < _numComponents; i++)
   {
      if (_products[i] && _products[i]->isVisible())
      {
         Components *q = _products[i]->getComponents();
         if (q && q->numComponents() && q->hasVisible())
         {
            ret = True;
         }
         else if (_products[i]->isSelected() == False || _products[i]->skipped())
         {
            ret = True;
         }
      }
      if (ret) break;

   }

   return ret;
}

static const char *product_order[] =
{
"svrcore",
"base",
"slapd",
"admin",
"cert",
"msg",
"mail",
"news",
"calendar",
"proxy",
"https",
"httpd",
"compass",
NULL
};

void
Components::sortComponents()
{
   Product **it, **list;
   Product *temp;
   int i = 0;
   const char **product_name = product_order;

   list = _products;

   /*
    * On the safe side first, find product_name and
    * move to the correct position based on (hard) dependency
    */
   while (*product_name != NULL)
   {
      for (it = list + i; *it && strcasecmp((*it)->get(PKG_NICKNAME), *product_name); ++it)
         ;
      if (*it)
      {
        if (it != list + i)
        {
           temp = *it;
           *it = *(list + i);
           *(list + i) = temp;
        }
        i++;
     }
     product_name++;
  }

  dependencySort(list);
}

NSString
Components::checkInstallDependency(Product *product, Components *oldComponents)
{
   NVPair *dp = NULL;
   NSString s;
   const char *r = NULL;

   /*
    * Go through the product list and determine whether all dependency
    * are satisfied, i.e. if a product depends on one or more products
    * the depended have to be selected
    */

   // Find where this product is in my list
   Product **where = NULL;
   for (int i = 0; i < _numComponents; i++)
   {
      if (!strcasecmp(_products[i]->get(PKG_NICKNAME), product->get(PKG_NICKNAME)))
      {
         where = _products + i;
      }
   }

   dp = product->dependencies(PKG_DEPENDENCIES);

   if (dp)
   {
      Product **p;
      int i = 0;

      s = dp->name(i);
      r = dp->get(s);

      while (s != (char *) NULL)
      {
         p = where - 1;
         while (p >= _products)
         {
            if (!strcasecmp(s, (*p)->get(PKG_NICKNAME)) &&
                //!strcasecmp(r, (*p)->get(PKG_VERSION)) &&
                (setupVersionCompare((*p)->get(PKG_VERSION),r)!=-1) &&
                ((*p)->downgrade() || (*p)->isSelected() || (*p)->installed() || !(*p)->isVisible() ||
                 ((*p)->exists() && (*p)->compatible())))
            {
               if (s = dp->name(++i))
                  r = dp->get(s);
               break;
            }
            else
            {
               --p;
            }
         }

         if (p < _products)
         {
            break;
         }
      }
   }

   if (s.data() != (char *) NULL) 
   {
      Product *q = oldComponents->getComponent(s.data());
      if (q != NULL)
      {
         const char *v = q->get(PKG_VERSION);
         if (v)
         {
            if (setupVersionCompare(v,r) != -1)
            {
               s = NULL;
            }
         }
      }
   }

   if (dp)
     delete dp;

   return s;
}


/*
 *  search valid product in current package
 *  return  n:  product index in components list
 *         -1:  can't find the product
 */

int Components::searchProduct(const char *name, const char *version)
{
   Product *p;
   int i;
  
   for (i = 0; i < numComponents(); i++)
   {
      p = getComponent(i);

      if (strcasecmp(name, p->get(PKG_NICKNAME)) == 0)
      {
         if (setupVersionCompare(p->get(PKG_VERSION),version) != -1)
            return i;
         else
            break;
      }
   }

   return -1;
}


/*
 * check product dependency within current package
 *  return  0:  product dependencies are satisfied
 *         -1:  product dependencies are not inside package
 *         -2:  dependency is not selected, set nickname to the dependency name
 */

int  
Components::checkProductDependencies(Product *product, char *nickname)
{
   NVPair *dp = NULL;
   NSString name;
   const char *version = NULL;
   int err = 0;
   char errMsg[BIG_BUF];
   int position;

   errMsg[0] = '\0';

   /*
    * Go through the product list and determine whether all dependency
    * are satisfied, i.e. if a product depends on one or more products
    * the depended have to be selected
    */

   // Find where this product is in my list
   Product **where = NULL;

   // product must in package list
   position = searchProduct(product->get(PKG_NICKNAME), product->get(PKG_VERSION));
   
   where = _products + position;
   

   dp = product->dependencies(PKG_DEPENDENCIES);

   if (dp)
   {
      Product *p;
      int i = 0;
      int n;

      while(name = dp->name(i++))
      {
         version = dp->get(name);

         if ((n = searchProduct(name, version)) == -1)
         {
            err = -1;         // not found
            strcpy(nickname, name);
            break;
         }

         p = getComponent(n);
         if (p->skipped() || p->isSelected() || (p->isVisible() == False) ||p->installed() ||p->downgrade())
         {  // keep going
            err = 0;
         }
         else
         {  
            err = -2;
            strcpy(nickname, name);
            break;
         }         
      }
   }

   if (dp)
     delete dp;

   return err;
}

void 
Components::selectInstallDependency()
{
   NVPair *dp = NULL;
   Product *p;

   for (int i = 0; i < _numComponents; i++)
   {
      p = _products[i];

      /*
       * select dependencies if not selected.
       * Note:
       *  since we did dependency check already on visible components
       *  should we go through dependency check on invisible components ?
       *  let's wait and see (cxue)
       */ 
      dp = p->dependencies(PKG_DEPENDENCIES);

      if (dp && p->isSelected() && !p->skipped())
      {
         for (int j = 0; j < dp->numName(); j++)
         {         
            selectComponent(dp->name(j));
         }
      }
   }
}

void 
Components::selectMandatoryComponents()
{
   Product *p;
   for (int i = 0; i < _numComponents; i++)
   {
      p = _products[i];
      /*
       * select invisible mandatory components
       */
      if (p->isVisible() == False && p->isMandatory() == True &&
          p->isChecked() == True && p->compatible() && !p->installed() && !p->downgrade())
      {
         p->select();
      }
   }         
}

int
dependencySort(Product **list)
{
   static int loop = 0;
   Product *temp;
   NVPair *dp = NULL;

   if (*list != NULL)
   {
      int next = 1;

      if (++loop > 50)
      {
/*
         if (installMode == Silent)
         {
            ins_log_message(FATAL, "Install", "Circular dependency det
ected for server %s.", (*list)->nickName);
            ins_log_message(FATAL, "Install", "Please contact Fedora 
 Tecnical Support.");
            printf(fatal_message, logFile);
            exit(1);
         }
         else
         {
            exit_message("ERROR: Circular dependency detected for serv
er %s\n.", (*list)->nickName);
         }
*/
         printf("ERROR\n");
         exit(1);
      }

      /* Go through dependency list until exhausted */

      dp = (*list)->dependencies(PKG_DEPENDENCIES);

      if (dp)
      {
         Product **p = list;
         const char *q = NULL;
         Product **k;
         int i = 0;

         /* Get first dependency */
         q = dp->name(i);
         while (q != NULL)
         {
            k = p;
            while (*k != NULL)
            {
               /* Find the one *list depends on */
               if (strcasecmp(q, (*k)->get(PKG_NICKNAME)))
               {
                  ++k;
               }
               else
               {
                  temp = *k;
                  *k = *p;
                  *p = temp;
                  p = k;
                  next = 0;
               }
            }
            q = dp->name(++i);
            p = list;
         }
      }

      if (next)
         dependencySort(list + 1);
      else
         dependencySort(list);
   }

   if (dp)
      delete dp;

   return 0;
}

void
Product::relocate(const char *serverRoot, NSString & prog) const
{
   const char *s;
   NSString newProg;

   if (prog[0] != '/')
   {
      prog = NSString(serverRoot) + "/" + prog;
   }

   // At this time prog is full path name

   s = strrchr(prog.data(), '/') + 1;

   newProg = NSString(serverRoot) + "/" + s + "-post";

   rename(prog.data(), newProg.data());

   prog = newProg;

}

long Product::requiredDiskSpace(const NSString & sourceDir, NVPair *installInfo)
{
  char *archive;
  NSString filePath;
  char *buffer;
  const char *p;

  p = getInfo(installInfo, PKG_ARCHIVE);
  if (p == NULL)
    return 0;

  buffer = strdup(p);

  archive = strtok(buffer, ", ");
  while (archive != (char *) NULL)
  {
    filePath = sourceDir + "/" + get("SourcePath") + "/" + archive;
    unzipInfo(filePath, installInfo);

    archive = strtok(NULL, ", ");
  }

  free(buffer);

  return getArchiveSize();
}

long 
Product::getArchiveSize()
{
  return _archiveSize;
}

void 
Product::setArchiveSize(long size)
{
  _archiveSize = size;
}
